<?php

if (!defined('ABSPATH')) {
    exit; // Direct access not allowed.
}

function register_smstide_cart_abandonment_settings() {
    register_setting('smstide_cart_abandonment_settings_group', 'smstide_enable_cart_abandonment');
    register_setting('smstide_cart_abandonment_settings_group', 'smstide_first_cart_abandonment_follow_up_message');
    register_setting('smstide_cart_abandonment_settings_group', 'smstide_first_cart_abandonment_follow_up_time');
    register_setting('smstide_cart_abandonment_settings_group', 'smstide_second_cart_abandonment_follow_up_message');
    register_setting('smstide_cart_abandonment_settings_group', 'smstide_second_cart_abandonment_follow_up_time');
    register_setting('smstide_cart_abandonment_settings_group', 'smstide_third_cart_abandonment_follow_up_message');
    register_setting('smstide_cart_abandonment_settings_group', 'smstide_third_cart_abandonment_follow_up_time');
}
add_action('admin_init', 'register_smstide_cart_abandonment_settings');

function cart_abandonment_settings() {
    $is_woo_otp_active = is_plugin_active('woo-otp/woo-otp.php'); // Adjust the path if necessary

    $enable_cart_abandonment = get_option('smstide_enable_cart_abandonment', 'no');
    $first_follow_up_message = get_option('smstide_first_cart_abandonment_follow_up_message', 'Hi [user_first_name], You have some great stuff in your Shopping Cart - don\'t wait too long! Complete your order now - our popular products go fast.');
    $first_follow_up_time = get_option('smstide_first_cart_abandonment_follow_up_time', '60');
    $second_follow_up_message = get_option('smstide_second_cart_abandonment_follow_up_message', 'Hi [user_first_name], You have some great stuff in your Shopping Cart - don\'t wait too long! Complete your order now - our popular products go fast.');
    $second_follow_up_time = get_option('smstide_second_cart_abandonment_follow_up_time', '1440');
    $third_follow_up_message = get_option('smstide_third_cart_abandonment_follow_up_message', 'Hi [user_first_name], You have some great stuff in your Shopping Cart - don\'t wait too long! Complete your order now - our popular products go fast.');
    $third_follow_up_time = get_option('smstide_third_cart_abandonment_follow_up_time', '2880');

    ?>
    <div class="wrap">
        <h2><?php _e('Cart Abandonment', 'woo-geolocation'); ?></h2>

        <form action="options.php" method="post">
            <?php settings_fields('smstide_cart_abandonment_settings_group'); ?>
            <?php do_settings_sections('smstide_cart_abandonment_settings_group'); ?>
            <table class="form-table">
                <tr>
                    <th><?php _e('Enable Cart Abandonment', 'woo-geolocation'); ?></th>
                    <td>
                        <input type="checkbox" name="smstide_enable_cart_abandonment" id="smstide_enable_cart_abandonment" value="yes" <?php checked($enable_cart_abandonment, 'yes'); ?> <?php echo $is_woo_otp_active ? '' : 'disabled'; ?> />
                        <?php _e('Enable this option to track and handle cart abandonment.', 'woo-geolocation'); ?>
                        <?php if (!$is_woo_otp_active) : ?>
                            <p style="color: red;"><?php _e('Woo OTP Plugin required to use smstide Cart Abandonment Service. Get it from here <a href="https://themeplace.net/product/woo-otp-plugin-for-woocommerce/">Woo OTP Plugin</a>.', 'woo-geolocation'); ?></p>
                        <?php endif; ?>
                    </td>
                </tr>
                <tr>
                    <th><?php _e('First SMS Follow Up Message', 'woo-geolocation'); ?></th>
                    <td>
                        <textarea name="smstide_first_cart_abandonment_follow_up_message" id="smstide_first_cart_abandonment_follow_up_message" cols="72" rows="5"><?php echo esc_textarea($first_follow_up_message); ?></textarea>
                    </td>
                </tr>
                <tr>
                    <th><?php _e('Second SMS Follow Up Message', 'woo-geolocation'); ?></th>
                    <td>
                        <textarea name="smstide_second_cart_abandonment_follow_up_message" id="smstide_second_cart_abandonment_follow_up_message" cols="72" rows="5"><?php echo esc_textarea($second_follow_up_message); ?></textarea>
                    </td>
                </tr>
                <tr>
                    <th><?php _e('Third SMS Follow Up Message', 'woo-geolocation'); ?></th>
                    <td>
                        <textarea name="smstide_third_cart_abandonment_follow_up_message" id="smstide_third_cart_abandonment_follow_up_message" cols="72" rows="5"><?php echo esc_textarea($third_follow_up_message); ?></textarea>
                        <p><?php _e('Use these shortcodes to customize your message: [user_first_name], [cart_page_url], [checkout_page_url], [cart_items].', 'woo-geolocation'); ?></p>
                    </td>
                </tr>
                <tr>
                    <th><?php _e('First Follow Up Time (in minutes)', 'woo-geolocation'); ?></th>
                    <td>
                        <input type="text" name="smstide_first_cart_abandonment_follow_up_time" id="smstide_first_cart_abandonment_follow_up_time" value="<?php echo esc_attr($first_follow_up_time); ?>" placeholder="<?php _e('Enter follow up minutes', 'woo-geolocation'); ?>" />
                    </td>
                </tr>
                <tr>
                    <th><?php _e('Second Follow Up Time (in minutes)', 'woo-geolocation'); ?></th>
                    <td>
                        <input type="text" name="smstide_second_cart_abandonment_follow_up_time" id="smstide_second_cart_abandonment_follow_up_time" value="<?php echo esc_attr($second_follow_up_time); ?>" placeholder="<?php _e('Enter follow up minutes', 'woo-geolocation'); ?>" />
                    </td>
                </tr>
                <tr>
                    <th><?php _e('Third Follow Up Time (in minutes)', 'woo-geolocation'); ?></th>
                    <td>
                        <input type="text" name="smstide_third_cart_abandonment_follow_up_time" id="smstide_third_cart_abandonment_follow_up_time" value="<?php echo esc_attr($third_follow_up_time); ?>" placeholder="<?php _e('Enter follow up minutes', 'woo-geolocation'); ?>" />
                    </td>
                </tr>
            </table>
            <?php submit_button(); ?>
        </form>
    </div>
    <?php
}

function schedule_follow_up_events($entry_id) {
    global $wpdb;
    
    // Check if the woo-otp plugin is active
    if (!is_plugin_active('woo-otp/woo-otp.php')) {
        return;
    }

    // Fetch the initial product added cart datetime
    $table_name = $wpdb->prefix . 'smstide_cart_abandonment';
    $entry = $wpdb->get_row($wpdb->prepare("SELECT * FROM $table_name WHERE id = %d", $entry_id));

    if (!$entry) {
        error_log("No entry found with ID: $entry_id");
        return;
    }

    // Check if mobile number is available
    $user_details = $wpdb->get_row($wpdb->prepare("SELECT * FROM {$wpdb->prefix}woootp_user_details WHERE user_id = %d", $entry->user_id), ARRAY_A);

    if (!$user_details || empty($user_details['mobile_number'])) {
        error_log("No mobile number found for user_id: $entry->user_id");
        return;
    }

    $product_added_cart_datetime = $entry->product_added_cart_datetime;

    // Fetch follow-up times from the wp_options
    $first_follow_up_time = get_option('smstide_first_cart_abandonment_follow_up_time', 60);
    $second_follow_up_time = get_option('smstide_second_cart_abandonment_follow_up_time', 1440);
    $third_follow_up_time = get_option('smstide_third_cart_abandonment_follow_up_time', 2880);

    // Calculate follow-up times based on initial product added cart datetime
    $product_added_time = strtotime($product_added_cart_datetime);

    if (!empty($first_follow_up_time) && $first_follow_up_time > 0) {
        $first_follow_up_timestamp = $product_added_time + ($first_follow_up_time * 60); // Convert minutes to seconds
        wp_schedule_single_event($first_follow_up_timestamp, 'send_cart_abandonment_follow_up', array($entry_id, 'first'));
    }

    if (!empty($second_follow_up_time) && $second_follow_up_time > 0) {
        $second_follow_up_timestamp = $product_added_time + ($second_follow_up_time * 60); // Convert minutes to seconds
        wp_schedule_single_event($second_follow_up_timestamp, 'send_cart_abandonment_follow_up', array($entry_id, 'second'));
    }

    if (!empty($third_follow_up_time) && $third_follow_up_time > 0) {
        $third_follow_up_timestamp = $product_added_time + ($third_follow_up_time * 60); // Convert minutes to seconds
        wp_schedule_single_event($third_follow_up_timestamp, 'send_cart_abandonment_follow_up', array($entry_id, 'third'));
    }

    // Update next notification datetime to the first follow-up time in UTC if it exists
    $next_notification_datetime = !empty($first_follow_up_time) && $first_follow_up_time > 0 ? gmdate('Y-m-d H:i:s', $first_follow_up_timestamp) : NULL;
    $wpdb->update(
        $wpdb->prefix . 'smstide_cart_abandonment',
        ['next_notification_datetime' => $next_notification_datetime],
        ['id' => $entry_id],
        ['%s'],
        ['%d']
    );
}

function clear_scheduled_events($entry_id) {
    $hooks = array('send_cart_abandonment_follow_up');
    foreach ($hooks as $hook) {
        $crons = _get_cron_array();
        if (is_array($crons)) {
            foreach ($crons as $timestamp => $cron) {
                if (isset($cron[$hook])) {
                    foreach ($cron[$hook] as $key => $scheduled) {
                        if ($scheduled['args'][0] == $entry_id) {
                            wp_unschedule_event($timestamp, $hook, $scheduled['args']);
                        }
                    }
                }
            }
        }
    }
}

function handle_cart_abandonment_add_to_cart($cart_item_key, $product_id, $quantity, $variation_id, $variation, $cart_item_data) {
    
    // Check if the woo-otp plugin is active
    if (!is_plugin_active('woo-otp/woo-otp.php')) {
        return;
    }
    
    if (is_user_logged_in()) {
        global $wpdb;

        $user_id = get_current_user_id();
        $current_time = current_time('mysql', 1); // Use UTC

        // Check if mobile number is available
        $user_details = $wpdb->get_row($wpdb->prepare("SELECT * FROM {$wpdb->prefix}woootp_user_details WHERE user_id = %d", $user_id), ARRAY_A);

        if (!$user_details || empty($user_details['mobile_number'])) {
            error_log("No mobile number found for user_id: $user_id");
            return;
        }

        $table_name = $wpdb->prefix . 'smstide_cart_abandonment';

        $existing_entry = $wpdb->get_row($wpdb->prepare(
            "SELECT * FROM $table_name WHERE user_id = %d",
            $user_id
        ));

        if ($existing_entry) {
            // Clear existing scheduled events
            clear_scheduled_events($existing_entry->id);

            $product_ids = maybe_unserialize($existing_entry->product_ids);
            if (!in_array($product_id, $product_ids)) {
                $product_ids[] = $product_id;
            }

            $updated = $wpdb->update(
                $table_name,
                [
                    'product_ids' => maybe_serialize($product_ids),
                    'last_notification_sent_datetime' => NULL,
                    'next_notification_datetime' => NULL,
                    'first_message_follow_up' => 0,
                    'second_message_follow_up' => 0,
                    'third_message_follow_up' => 0,
                    'product_added_cart_datetime' => $current_time
                ],
                [
                    'user_id' => $user_id
                ],
                [
                    '%s', '%s', '%s', '%d', '%d', '%d', '%s'
                ],
                [
                    '%d'
                ]
            );

            if ($updated !== false) {
                schedule_follow_up_events($existing_entry->id);
            }
        } else {
            $product_ids = array($product_id);
            $inserted = $wpdb->insert(
                $table_name,
                [
                    'user_id' => $user_id,
                    'product_ids' => maybe_serialize($product_ids),
                    'first_message_follow_up' => 0,
                    'second_message_follow_up' => 0,
                    'third_message_follow_up' => 0,
                    'product_added_cart_datetime' => $current_time
                ],
                [
                    '%d', '%s', '%d', '%d', '%d', '%s'
                ]
            );

            if ($inserted !== false) {
                schedule_follow_up_events($wpdb->insert_id);
            }
        }
    }
}
add_action('woocommerce_add_to_cart', 'handle_cart_abandonment_add_to_cart', 10, 6);

function send_cart_abandonment_follow_up($entry_id, $follow_up_type) {
    global $wpdb;
    
    // Check if the woo-otp plugin is active
    if (!is_plugin_active('woo-otp/woo-otp.php')) {
        return;
    }

    $table_name = $wpdb->prefix . 'smstide_cart_abandonment';
    $entry = $wpdb->get_row($wpdb->prepare("SELECT * FROM $table_name WHERE id = %d", $entry_id));

    if (!$entry) {
        error_log("No entry found with ID: $entry_id");
        return;
    }

    $current_time = current_time('mysql', 1); // Use UTC

    $user_id = $entry->user_id;
    $product_ids = maybe_unserialize($entry->product_ids);
    $product_added_cart_datetime = $entry->product_added_cart_datetime;

    if (empty($product_ids)) {
        error_log("No product_ids found for entry_id: $entry_id");
        return;
    }

    $first_follow_up_message = get_option('smstide_first_cart_abandonment_follow_up_message', 'Hi [user_first_name], You have some great stuff in your Shopping Cart - don/t wait too long! Complete your order now - our popular products go fast.');
    $second_follow_up_message = get_option('smstide_second_cart_abandonment_follow_up_message', 'Hi [user_first_name], You have some great stuff in your Shopping Cart - don/t wait too long! Complete your order now - our popular products go fast.');
    $third_follow_up_message = get_option('smstide_third_cart_abandonment_follow_up_message', 'Hi [user_first_name], You have some great stuff in your Shopping Cart - don/t wait too long! Complete your order now - our popular products go fast.');

    $user_details = $wpdb->get_row($wpdb->prepare("SELECT * FROM {$wpdb->prefix}woootp_user_details WHERE user_id = %d", $user_id), ARRAY_A);

    if (!$user_details || empty($user_details['mobile_number'])) {
        error_log("No mobile number found for user_id: $user_id");
        return;
    }

    $user_first_name = $user_details['first_name'];
    $mobile_number = $user_details['mobile_number'];

    $default_messages = [
        'first' => $first_follow_up_message,
        'second' => $second_follow_up_message,
        'third' => $third_follow_up_message
    ];

    $message_template = $default_messages[$follow_up_type];

    $last_product_id = end($product_ids);
    $product_image_url = wp_get_attachment_url(get_post_thumbnail_id($last_product_id));

    // Replace shortcodes with actual values
    $cart_page_url = wc_get_cart_url();
    $checkout_page_url = wc_get_checkout_url();

    // Get product names from the cart
    $product_names = array();
    foreach ($product_ids as $product_id) {
        $product = wc_get_product($product_id);
        if ($product) {
            $product_names[] = $product->get_name();
        }
    }
    $cart_items = implode("\n", $product_names);

    $message = str_replace(
        ['[user_first_name]', '[cart_page_url]', '[checkout_page_url]', '[cart_items]'],
        [$user_first_name, $cart_page_url, $checkout_page_url, $cart_items],
        $message_template
    );

    $messagingsms_settings = get_option('messagingsms_setting', '');

    if (!empty($messagingsms_settings)) {
        $messagingsms_settings = maybe_unserialize($messagingsms_settings);

        $smstide_api_key = $messagingsms_settings['messagingsms_woocommerce_api_key'];
        $sending_service = $messagingsms_settings['messagingsms_woocommerce_service'];
        $smstide_whatsapp_id = $messagingsms_settings['messagingsms_woocommerce_whatsapp'];
        $smstide_device_id = $messagingsms_settings['messagingsms_woocommerce_device'];
        $smstide_sim_slot = $messagingsms_settings['messagingsms_woocommerce_sim'];
    } else {
        error_log('Failed to retrieve messaging SMS settings');
        return;
    }

    if ($sending_service === '2') { // WhatsApp
        $notification = [
            "secret" => $smstide_api_key,
            "account" => $smstide_whatsapp_id,
            "type" => "media",
            "media_type" => "image",
            "media_url" => $product_image_url,
            "recipient" => $mobile_number,
            "message" => $message
        ];
        $api_url = "https://smstide.com/api/send/whatsapp";
    } else { // SMS
        $notification = [
            "secret" => $smstide_api_key,
            "mode" => "devices",
            "phone" => $mobile_number,
            "device" => $smstide_device_id,
            "sim" => $smstide_sim_slot,
            "message" => $message,
        ];
        $api_url = "https://smstide.com/api/send/sms";
    }

    $cURL = curl_init($api_url);
    curl_setopt($cURL, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($cURL, CURLOPT_POSTFIELDS, $notification);
    $response = curl_exec($cURL);
    if (curl_errno($cURL)) {
        error_log('cURL error: ' . curl_error($cURL));
    } else {
        error_log("Notification sent with response: $response");
    }
    curl_close($cURL);

    $wpdb->update(
        $table_name,
        [
            'last_notification_sent_datetime' => $current_time,
            "{$follow_up_type}_message_follow_up" => 1
        ],
        [
            'id' => $entry_id
        ],
        [
            '%s', '%d'
        ],
        [
            '%d'
        ]
    );

    // Calculate and update next notification datetime for the next follow-up
    $next_follow_up_type = '';
    $product_added_time = strtotime($product_added_cart_datetime);

    if ($follow_up_type == 'first') {
        $next_follow_up_type = 'second';
        $next_follow_up_time = get_option('smstide_second_cart_abandonment_follow_up_time', 1440);
    } elseif ($follow_up_type == 'second') {
        $next_follow_up_type = 'third';
        $next_follow_up_time = get_option('smstide_third_cart_abandonment_follow_up_time', 2880);
    }

    if (!empty($next_follow_up_time) && $next_follow_up_time > 0) {
        $next_notification_time = $product_added_time + ($next_follow_up_time * 60); // Convert minutes to seconds
        $wpdb->update(
            $table_name,
            ['next_notification_datetime' => gmdate('Y-m-d H:i:s', $next_notification_time)],
            ['id' => $entry_id],
            ['%s'],
            ['%d']
        );
    } else {
        // Check if there are no more follow-up events scheduled
        $remaining_events = check_remaining_events($entry_id);
        if (!$remaining_events) {
            clear_scheduled_events($entry_id);
            $wpdb->delete($table_name, ['id' => $entry_id], ['%d']);
        }
    }
}
add_action('send_cart_abandonment_follow_up', 'send_cart_abandonment_follow_up', 10, 2);

function check_remaining_events($entry_id) {
    $hooks = array('send_cart_abandonment_follow_up');
    foreach ($hooks as $hook) {
        $crons = _get_cron_array();
        if (is_array($crons)) {
            foreach ($crons as $timestamp => $cron) {
                if (isset($cron[$hook])) {
                    foreach ($cron[$hook] as $key => $scheduled) {
                        if ($scheduled['args'][0] == $entry_id) {
                            return true;
                        }
                    }
                }
            }
        }
    }
    return false;
}

function handle_order_placed($order_id) {
    global $wpdb;

    $order = wc_get_order($order_id);
    $user_id = $order->get_user_id();
    $table_name = $wpdb->prefix . 'smstide_cart_abandonment';

    foreach ($order->get_items() as $item) {
        $product_id = $item->get_product_id();

        $entry = $wpdb->get_row($wpdb->prepare("SELECT * FROM $table_name WHERE user_id = %d", $user_id));

        if ($entry) {
            $product_ids = maybe_unserialize($entry->product_ids);

            if (($key = array_search($product_id, $product_ids)) !== false) {
                unset($product_ids[$key]);
            }

            if (empty($product_ids)) {
                // Clear scheduled events before deleting the entry
                clear_scheduled_events($entry->id);

                $wpdb->delete($table_name, ['id' => $entry->id], ['%d']);
            } else {
                $wpdb->update(
                    $table_name,
                    ['product_ids' => maybe_serialize($product_ids)],
                    ['id' => $entry->id],
                    ['%s'],
                    ['%d']
                );
            }
        }
    }
}
add_action('woocommerce_checkout_order_processed', 'handle_order_placed');

function handle_cart_abandonment_remove_from_cart($cart_item_key, $cart) {
    if (is_user_logged_in()) {
        global $wpdb;

        $user_id = get_current_user_id();
        $product_id = $cart->removed_cart_contents[$cart_item_key]['product_id'];

        $table_name = $wpdb->prefix . 'smstide_cart_abandonment';
        $entry = $wpdb->get_row($wpdb->prepare("SELECT * FROM $table_name WHERE user_id = %d", $user_id));

        if ($entry) {
            $product_ids = maybe_unserialize($entry->product_ids);

            if (($key = array_search($product_id, $product_ids)) !== false) {
                unset($product_ids[$key]);
            }

            if (empty($product_ids)) {
                // Clear scheduled events before deleting the entry
                clear_scheduled_events($entry->id);

                $wpdb->delete($table_name, ['id' => $entry->id], ['%d']);
            } else {
                $wpdb->update(
                    $table_name,
                    ['product_ids' => maybe_serialize($product_ids)],
                    ['id' => $entry->id],
                    ['%s'],
                    ['%d']
                );
            }
        }
    }
}
add_action('woocommerce_cart_item_removed', 'handle_cart_abandonment_remove_from_cart', 10, 2);