<?php

if (!defined('ABSPATH')) {
    exit; // Direct access not allowed.
}

function smstide_register_fake_order_settings() {
    // Prevent Fake Orders Settings
    register_setting('smstide_prevent_fake_orders_group', 'smstide_enable_fake_order_prevention');
    register_setting('smstide_prevent_fake_orders_group', 'smstide_enable_fake_order_prevention_cod');
    register_setting('smstide_prevent_fake_orders_group', 'smstide_prevent_fake_order_otp_message');
}
add_action('admin_init', 'smstide_register_fake_order_settings');

function fake_orders_settings() {
    $prevent_fake_orders = get_option('smstide_enable_fake_order_prevention', 'no');
    $prevent_fake_orders_cod = get_option('smstide_enable_fake_order_prevention_cod', 'no');
    $otp_message = get_option('smstide_prevent_fake_order_otp_message', 'Your OTP is {{otp}}');

    ?>
    <div class="wrap">
        <h2><?php _e('smstide Prevent Fake Orders', 'woo-geolocation'); ?></h2>

        <form action="options.php" method="post">
            
            <?php settings_fields('smstide_prevent_fake_orders_group'); ?>
            <?php do_settings_sections('smstide_prevent_fake_orders_group'); ?>
            
            <table class="form-table">
                <tr valign="top">
                    <th scope="row"><?php _e('Enable Settings', 'woo-geolocation'); ?></th>
                    <td>
                        <input type="checkbox" name="smstide_enable_fake_order_prevention" value="yes" <?php checked($prevent_fake_orders, 'yes'); ?> />
                        <?php _e('By enabling this option you may prevent fake orders on the website by verifying the user\'s mobile number using OTP.', 'woo-geolocation'); ?>
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row"><?php _e('Enable for COD Orders Only', 'woo-geolocation'); ?></th>
                    <td>
                        <input type="checkbox" name="smstide_enable_fake_order_prevention_cod" value="yes" <?php checked($prevent_fake_orders_cod, 'yes'); ?> />
                        <?php _e('Only verify OTP for COD payment method. Do not enable this option if you want to verify OTP for all payment methods.', 'woo-geolocation'); ?>
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row"><?php _e('Custom OTP Message', 'woo-geolocation'); ?></th>
                    <td>
                        <input type="text" name="smstide_prevent_fake_order_otp_message" value="<?php echo esc_attr($otp_message); ?>" class="regular-text" />
                        <p class="description">Enter the custom OTP message. Use {{otp}} as a placeholder for the OTP code.</p>
                    </td>
                </tr>
            </table>
            <?php submit_button(); ?>
        </form>
    </div>
    <?php
}

function smstide_checkout_page_order_otp_modal() {
    ?>
    <!-- HTML for the OTP popup -->
    <div id="checkout-page-orders-otp-popup" style="display: none;">
        <div id="checkout-otp-popup-container">
            <div class="checkout-otp-popup-title">Verify your mobile number</div>
            <input type="text" id="checkout-order-otp-input" class="checkout-order-enter-otp-input" placeholder="Enter OTP" />
            <button id="checkout-order-verify-otp-btn">Verify OTP</button>
            <div id="checkout-order-incorrect-otp-error"></div>
        </div>
    </div>
    <!-- End of HTML for the OTP popup -->
    <?php
}
add_action('wp_footer', 'smstide_checkout_page_order_otp_modal');

function smstide_send_prevent_fake_order_otp() {
    // Nonce verification
    if (!check_ajax_referer('smstide_front_nonce_action', 'nonce', false)) {
        wp_send_json_error(array('message' => 'Nonce verification failed.'));
        die();
    }

    // Get messaging SMS settings
    $messagingsms_settings = get_option('messagingsms_setting', '');

    if (!empty($messagingsms_settings)) {
        $messagingsms_settings = maybe_unserialize($messagingsms_settings);

        $smstide_api_key = $messagingsms_settings['messagingsms_woocommerce_api_key'];
        $sending_service = $messagingsms_settings['messagingsms_woocommerce_service'];
        $smstide_whatsapp_id = $messagingsms_settings['messagingsms_woocommerce_whatsapp'];
        $smstide_device_id = $messagingsms_settings['messagingsms_woocommerce_device'];
        $smstide_sim_slot = $messagingsms_settings['messagingsms_woocommerce_sim'];
    } else {
        error_log('Failed to retrieve messaging SMS settings');
        return;
    }

    // Get the country code and phone number
    $phone_number = sanitize_text_field($_POST['phone_number']);
    $billing_country = sanitize_text_field($_POST['billing_country']);

    $view = new MessagingSMS_SendSMS_View();
    $countryList = $view->mapi_getCountryList();
    $dial_codes = array_column($countryList, 'd_code', 'code');
    
    $country_code = isset($dial_codes[$billing_country]) ? $dial_codes[$billing_country] : '';
    $formatted_phone = $country_code . ltrim($phone_number, '0');

    if (empty($formatted_phone)) {
        wp_send_json_error(array('message' => 'Invalid country code.'));
        return;
    }

    // Send OTP
    $smstide_otp_message = get_option('smstide_prevent_fake_order_otp_message', 'Your OTP is {{otp}}');

    if ($sending_service === '2') {
        $message = [
            "secret" => $smstide_api_key,
            "type" => "whatsapp",
            "account" => $smstide_whatsapp_id,
            "phone" => $formatted_phone,
            "message" => empty($smstide_otp_message) ? 'Your OTP is {{otp}}' : $smstide_otp_message
        ];
    } else {
        $message = [
            "secret" => $smstide_api_key,
            "type" => "sms",
            "mode" => "devices",
            "device" => $smstide_device_id,
            "sim" => $smstide_sim_slot,
            "phone" => $formatted_phone,
            "message" => empty($smstide_otp_message) ? 'Your OTP is {{otp}}' : $smstide_otp_message
        ];
    }

    $cURL = curl_init("https://smstide.com/api/send/otp");
    curl_setopt($cURL, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($cURL, CURLOPT_POSTFIELDS, $message);
    $response = curl_exec($cURL);
    $error = curl_error($cURL);
    curl_close($cURL);

    $result = json_decode($response, true);

    if ($error) {
        wp_send_json_error(array('message' => 'Failed to send OTP due to cURL error. Please try again.'));
    } elseif ($result && isset($result['status']) && $result['status'] === 200) {
        wp_send_json_success(array('message' => 'OTP sent successfully.'));
    } else {
        wp_send_json_error(array('message' => isset($result['message']) ? $result['message'] : 'Failed to send OTP. Please try again.'));
    }
}
add_action('wp_ajax_smstide_send_prevent_fake_order_otp', 'smstide_send_prevent_fake_order_otp');
add_action('wp_ajax_nopriv_smstide_send_prevent_fake_order_otp', 'smstide_send_prevent_fake_order_otp');

function smstide_checkout_otp_verification() {
    // Nonce verification
    if (!check_ajax_referer('smstide_front_nonce_action', 'nonce', false)) {
        wp_send_json_error(array('message' => 'Nonce verification failed.'));
        die();
    }

    $messagingsms_settings = get_option('messagingsms_setting', '');

    if (!empty($messagingsms_settings)) {
        $messagingsms_settings = maybe_unserialize($messagingsms_settings);

        $smstide_api_key = $messagingsms_settings['messagingsms_woocommerce_api_key'];
        $sending_service = $messagingsms_settings['messagingsms_woocommerce_service'];
        $smstide_whatsapp_id = $messagingsms_settings['messagingsms_woocommerce_whatsapp'];
        $smstide_device_id = $messagingsms_settings['messagingsms_woocommerce_device'];
        $smstide_sim_slot = $messagingsms_settings['messagingsms_woocommerce_sim'];
        $apiSecret = $smstide_api_key;
    } else {
        wp_send_json_error(array('message' => 'Failed to retrieve messaging SMS settings.'));
        die();
    }

    $otp = sanitize_text_field($_POST['otp']);
    
    $cURL = curl_init();
    curl_setopt($cURL, CURLOPT_URL, "https://smstide.com/api/get/otp?secret={$apiSecret}&otp={$otp}");
    curl_setopt($cURL, CURLOPT_RETURNTRANSFER, true);
    $response = curl_exec($cURL);
    curl_close($cURL);

    $result = json_decode($response, true);

    if (!$result || !isset($result['status']) || $result['status'] !== 200) {
        wp_send_json_error(array('message' => 'OTP verification failed. Please try again.'));
        wp_die();
    } else {
        wp_send_json_success(array('message' => 'OTP verified successfully.'));
        wp_die();
    }
}
add_action('wp_ajax_smstide_checkout_otp_verification', 'smstide_checkout_otp_verification');
add_action('wp_ajax_nopriv_smstide_checkout_otp_verification', 'smstide_checkout_otp_verification');
