jQuery(function($) {
    var isMobileDeviceCheckoutPopup = window.innerWidth <= 480;

    // Function to log the current selected payment method
    function logSelectedPaymentMethod() {
        var currentPaymentMethod = $('input[name="payment_method"]:checked').val();
        if (currentPaymentMethod) {
            console.log('Selected payment method:', currentPaymentMethod);
        } else {
            console.log('No payment method selected.');
        }
    }

    // Using WooCommerce's `updated_checkout` event to detect when the checkout is refreshed
    $(document.body).on('updated_checkout', function() {
        logSelectedPaymentMethod();
    });

    // Use a setTimeout as a backup method
    setTimeout(logSelectedPaymentMethod, 1000);  // wait for 1 second

    // Using event delegation to detect the change of payment method
    $(document).on('change', 'input[name="payment_method"]', function() {
        logSelectedPaymentMethod();
    });

    // Function to show the OTP popup
    function showCheckoutOTPPopup() {
        $('#checkout-page-orders-otp-popup').css('display', 'flex');  // Always set to flex
    
        if (isMobileDeviceCheckoutPopup) {
            setTimeout(function() {
                $('#checkout-otp-popup-container').addClass('active');
            }, 100); // Short delay for smooth transition
        }
    }
    
    // Function to hide the OTP popup
    function hideCheckoutOTPPopup() {
        $('#checkout-otp-popup-container').removeClass('active');
        setTimeout(function() {
            $('#checkout-page-orders-otp-popup').css('display', 'none'); // Always set to none after transition
        }, 300); // Wait for the transition to finish (if any) before hiding
    }

    if (smstide_front_vars.enable_prevent_fake_orders === 'yes') {
        var otpVerified = false;

        $('form.checkout').on('submit', function(e) {
            // Check selected payment method
            var selectedPaymentMethod = $('input[name="payment_method"]:checked').val();
            
            // Check if we should interrupt the checkout process based on the selected payment method and your settings
            if(smstide_front_vars.enable_prevent_fake_orders_cod === 'yes' && selectedPaymentMethod !== 'cod') {
                return; // Continue with the checkout since OTP is only for COD and the selected payment method is not COD.
            }

            if (!otpVerified) {
                e.stopPropagation();
                e.stopImmediatePropagation();
                e.preventDefault();

                var $placeOrderBtn = $(this).find('button[name="woocommerce_checkout_place_order"]');
                $placeOrderBtn.prop('disabled', true);

                console.log("Place Order button intercepted");

                setTimeout(function() {
                    var phoneNumber = $('#billing_phone').val();
                    var country = $('#billing_country').val();

                    $.ajax({
                        url: smstide_front_vars.ajaxurl,
                        method: 'POST',
                        data: {
                            action: 'smstide_send_prevent_fake_order_otp',
                            nonce: smstide_front_vars.nonce,
                            phone_number: phoneNumber,
                            billing_country: country
                        },
                        success: function(response) {
                            console.log("OTP send response: ", response);
                            if (response.success) {
                                showCheckoutOTPPopup(); // Show the OTP popup
                            } else {
                                $placeOrderBtn.prop('disabled', false);
                                alert('Failed to send OTP. Please try again.');
                            }
                        }
                    });

                }, 1000);
            }
        });

        // Code for OTP verification when 'Verify' button is clicked
        $('#checkout-order-verify-otp-btn').on('click', function(e) {
            var enteredOtp = $('#checkout-order-otp-input').val();
            var phoneNumber = $('#billing_phone').val();

            $.ajax({
                url: smstide_front_vars.ajaxurl,
                method: 'POST',
                data: {
                    action: 'smstide_checkout_otp_verification',
                    nonce: smstide_front_vars.nonce,
                    phone_number: phoneNumber,
                    otp: enteredOtp
                },
                success: function(response) {
                    console.log("OTP verification response: ", response);
                    if (response.success) {
                        otpVerified = true;
                        hideCheckoutOTPPopup();
                        $('form.checkout').trigger('submit'); // Trigger checkout form submission again
                    } else {
                        alert('Incorrect OTP. Please try again.');
                    }
                }
            });
        });
    }
});
