<?php

/*
Plugin Name: SMSTide
Plugin URI:  https://smstide.com
Description: Send SMS for your WooCommerce Orders
Version:     3.7
Author:      SMSTide
Author URI:  https://smstide.com
License:     GPLv3
License URI: http://www.gnu.org/licenses/gpl-3.0.html
Text Domain: smstide-woocommerce
*/

use MessagingAPI_WC\Loader;

if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! defined( 'ZMPLGXX_NAME' ) ) {
	define("ZMPLGXX_NAME", "SMSTide");
}

if ( ! defined( 'ZMPLGXX_SITENAME' ) ) {
	define("ZMPLGXX_SITENAME", "SMSTide");
}

if ( ! defined( 'ZMPLGXX_URL' ) ) {
	define("ZMPLGXX_URL", "https://smstide.com");
}

if ( ! defined( 'ZMPLGXX_DOMAIN' ) ) {
	define("ZMPLGXX_DOMAIN", "smstide-woocommerce");
}

define("MESSAGINGSMS_PLUGIN_URL", plugin_dir_url(__FILE__));
define("MESSAGINGSMS_PLUGIN_DIR", plugin_dir_path(__FILE__));
define("MESSAGINGSMS_INC_DIR", MESSAGINGSMS_PLUGIN_DIR . "includes/");
define("MESSAGINGSMS_ADMIN_VIEW", MESSAGINGSMS_PLUGIN_DIR . "admin/");
define("MESSAGINGSMS_TEXT_DOMAIN", ZMPLGXX_DOMAIN);

require_once MESSAGINGSMS_PLUGIN_DIR . 'lib/action-scheduler/action-scheduler.php';
require_once MESSAGINGSMS_PLUGIN_DIR . 'custom/admin/smstide-cart-abandonment.php';
require_once MESSAGINGSMS_PLUGIN_DIR . 'custom/admin/smstide-prevent-fake-orders.php';

add_action( 'plugins_loaded', 'messagingsms_woocommerce_init', PHP_INT_MAX );

function messagingsms_woocommerce_init() {
    require_once(plugin_dir_path(__FILE__) . '/vendor/autoload.php');
	require_once ABSPATH . '/wp-admin/includes/plugin.php';
	require_once ABSPATH . '/wp-includes/pluggable.php';
	require_once MESSAGINGSMS_PLUGIN_DIR . 'interfaces/Messagingsms_PluginInterface.php';
	require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/contracts/class-messagingsms-register-interface.php';
	require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/class-messagingsms-helper.php';
	require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/class-messagingsms-woocommerce-frontend-scripts.php';
	require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/class-messagingsms-woocommerce-hook.php';
	require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/class-messagingsms-woocommerce-register.php';
	require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/class-messagingsms-woocommerce-logger.php';
	require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/class-messagingsms-woocommerce-notification.php';
	require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/class-messagingsms-download-log.php';
	require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/class-messagingsms-sendsms.php';
	require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/multivendor/class-messagingsms-multivendor.php';
	require_once MESSAGINGSMS_PLUGIN_DIR . 'lib/MessagingSMS.php';
	require_once MESSAGINGSMS_PLUGIN_DIR . 'lib/class.settings-api.php';
	require_once MESSAGINGSMS_PLUGIN_DIR . 'admin/class-messagingsms-woocommerce-setting.php';
	require_once MESSAGINGSMS_PLUGIN_DIR . 'admin/sendsms.php';
	require_once MESSAGINGSMS_PLUGIN_DIR . 'admin/automation.php';
	require_once MESSAGINGSMS_PLUGIN_DIR . 'admin/logs.php';
    require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/plugins/MessagingS2Member.php';
    require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/plugins/MessagingARMemberLite.php';
    require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/plugins/MessagingARMemberPremium.php';
    require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/plugins/MessagingMemberPress.php';
    require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/plugins/MessagingMemberMouse.php';
    require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/plugins/MessagingSimpleMembership.php';
    require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/plugins/MessagingRestaurantReservation.php';
    require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/plugins/MessagingQuickRestaurantReservation.php';
    require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/plugins/MessagingBookIt.php';
    require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/plugins/MessagingLatePoint.php';
    require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/plugins/MessagingFATService.php';
    require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/plugins/MessagingWpERP.php';
    require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/plugins/MessagingJetpackCRM.php';
    require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/plugins/MessagingFluentCRM.php';
    require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/plugins/MessagingGroundhoggCRM.php';
    require_once MESSAGINGSMS_PLUGIN_DIR . 'includes/plugins/MessagingSupportedPlugin.php';

    // load all Forms integrations
    Loader::load();

	//create notification instance
	$messagingsms_notification = new Messagingsms_WooCommerce_Notification();

	//register hooks and settings
	$registerInstance = new Messagingsms_WooCommerce_Register();
	$registerInstance->add( new Messagingsms_WooCommerce_Hook( $messagingsms_notification ) )
	                 ->add( new Messagingsms_WooCommerce_Setting() )
	                 ->add( new Messagingsms_WooCommerce_Frontend_Scripts() )
	                 ->add( new Messagingsms_Multivendor() )
	                 ->add( new Messagingsms_Download_log() )
	                 ->add( new MessagingSMS_SendSMS_View() )
	                 ->add( new MessagingSMS_Automation_View() )
	                 ->add( new MessagingSMS_Logs_View() )
	                 ->load();
}

// Constant values
define('smstide_VERSION', '3.7');

function smstide_enqueue_frontend_scripts() {
    wp_enqueue_script('smstide-front-script', MESSAGINGSMS_PLUGIN_URL . 'custom/assets/js/smstide-front-script.js', array('jquery'), smstide_VERSION, true);
    wp_enqueue_style('smstide-front-style', MESSAGINGSMS_PLUGIN_URL . 'custom/assets/css/smstide-front-style.css', array(), smstide_VERSION);
    
    // Pass data to the prevent fake order script
    $prevent_fake_orders = get_option('smstide_enable_fake_order_prevention', 'no');
    $prevent_fake_orders_cod = get_option('smstide_enable_fake_order_prevention_cod', 'no');

    // Pass data to the frontend script
    wp_localize_script('smstide-front-script', 'smstide_front_vars', array(
        'ajaxurl' => admin_url('admin-ajax.php'),
        'siteurl' => get_site_url(),
        'nonce' => wp_create_nonce('smstide_front_nonce_action'),
        'user_id' => get_current_user_id(),
        'enable_prevent_fake_orders' => $prevent_fake_orders,
        'enable_prevent_fake_orders_cod' => $prevent_fake_orders_cod
    ));
}

add_action('wp_enqueue_scripts', 'smstide_enqueue_frontend_scripts');

function smstide_enqueue_admin_scripts($hook) {

    wp_enqueue_script('smstide-admin-script', MESSAGINGSMS_PLUGIN_URL . 'custom/assets/js/smstide-admin-script.js', array('jquery'), smstide_VERSION, true);
    wp_enqueue_style('smstide-admin-style', MESSAGINGSMS_PLUGIN_URL . 'custom/assets/css/smstide-admin-style.css', array(), smstide_VERSION);

    // Pass data to the frontend script
    wp_localize_script('smstide-admin-script', 'smstide_admin_vars', array(
        'ajaxurl' => admin_url('admin-ajax.php'),
        'siteurl' => get_site_url(),
        'nonce' => wp_create_nonce('smstide_admin_nonce_action'),
    ));
}
add_action('admin_enqueue_scripts', 'smstide_enqueue_admin_scripts');

// Activation hook for creating a new table
register_activation_hook(__FILE__, 'smstide_plugin_activation');

function smstide_plugin_activation() {
    global $wpdb;
    
    // Create the directory for storing icon images if it doesn't exist
    $smstide_images_dir = WP_CONTENT_DIR . '/uploads/smstide/images/';
    if (!file_exists($smstide_images_dir)) {
        wp_mkdir_p($smstide_images_dir);
        chmod($smstide_images_dir, 0755);
    }

    // Create the cart abandonment table
    $cart_abandonment_table_name = $wpdb->prefix . 'smstide_cart_abandonment';
    
    if ($wpdb->get_var("SHOW TABLES LIKE '$cart_abandonment_table_name'") != $cart_abandonment_table_name) {
        $cart_abandonment = "CREATE TABLE $cart_abandonment_table_name (
            id mediumint(9) NOT NULL AUTO_INCREMENT,
            user_id bigint(20) NOT NULL,
            product_ids text NOT NULL,
            product_added_cart_datetime datetime DEFAULT NULL,
            first_message_follow_up tinyint(1) NOT NULL DEFAULT 0,
            second_message_follow_up tinyint(1) NOT NULL DEFAULT 0,
            third_message_follow_up tinyint(1) NOT NULL DEFAULT 0,
            last_notification_sent_datetime datetime DEFAULT NULL,
            next_notification_datetime datetime DEFAULT NULL,
            PRIMARY KEY  (id)
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8;";
    
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($cart_abandonment);
    }
}